@@ -1,7 +1,7 @@
 #
 # The internetarchive module is a Python/CLI interface to Archive.org.
 #
-# Copyright (C) 2012-2021 Internet Archive
+# Copyright (C) 2012-2024 Internet Archive
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -20,7 +20,7 @@
 internetarchive.item
 ~~~~~~~~~~~~~~~~~~~~
 
-:copyright: (C) 2012-2021 by Internet Archive.
+:copyright: (C) 2012-2024 by Internet Archive.
 :license: AGPL 3, see LICENSE for more details.
 """
 from __future__ import annotations
@@ -57,7 +57,7 @@ from internetarchive.utils import (
     iter_directory,
     json,
     norm_filepath,
-    recursive_file_count,
+    recursive_file_count_and_size,
     validate_s3_identifier,
 )
 
@@ -589,6 +589,7 @@ class Item(BaseItem):
                  verbose: bool = False,
                  ignore_existing: bool = False,
                  checksum: bool = False,
+                 checksum_archive: bool = False,
                  destdir: str | None = None,
                  no_directory: bool = False,
                  retries: int | None = None,
@@ -627,6 +628,10 @@ class Item(BaseItem):
 
         :param checksum: Skip downloading file based on checksum.
 
+        :param checksum_archive: Skip downloading file based on checksum, and skip
+                                 checksum validation if it already succeeded
+                                 (will create and use _checksum_archive.txt).
+
         :param destdir: The directory to download files to.
 
         :param no_directory: Download files to current working
@@ -670,6 +675,7 @@ class Item(BaseItem):
         ignore_existing = bool(ignore_existing)
         ignore_errors = bool(ignore_errors)
         checksum = bool(checksum)
+        checksum_archive = bool(checksum_archive)
         no_directory = bool(no_directory)
         return_responses = bool(return_responses)
         no_change_timestamp = bool(no_change_timestamp)
@@ -746,8 +752,8 @@ class Item(BaseItem):
                 ors = True
             else:
                 ors = False
-            r = f.download(path, verbose, ignore_existing, checksum, destdir,
-                           retries, ignore_errors, fileobj, return_responses,
+            r = f.download(path, verbose, ignore_existing, checksum, checksum_archive,
+                           destdir, retries, ignore_errors, fileobj, return_responses,
                            no_change_timestamp, params, None, stdout, ors, timeout)
             if return_responses:
                 responses.append(r)
@@ -770,6 +776,7 @@ class Item(BaseItem):
                         metadata: Mapping,
                         target: str | None = None,
                         append: bool = False,
+                        expect: Mapping | None = None,
                         append_list: bool = False,
                         insert: bool = False,
                         priority: int = 0,
@@ -777,7 +784,9 @@ class Item(BaseItem):
                         secret_key: str | None = None,
                         debug: bool = False,
                         headers: Mapping | None = None,
-                        request_kwargs: Mapping | None = None) -> Request | Response:
+                        request_kwargs: Mapping | None = None,
+                        timeout: int | float | None = None,
+                        refresh: bool = True) -> Request | Response:
         """Modify the metadata of an existing item on Archive.org.
 
         Note: The Metadata Write API does not yet comply with the
@@ -793,9 +802,14 @@ class Item(BaseItem):
         :param append: Append value to an existing multi-value
                        metadata field.
 
+        :param expect: Provide a dict of expectations to be tested
+                       server-side before applying patch to item metadata.
+
         :param append_list: Append values to an existing multi-value
                             metadata field. No duplicate values will be added.
 
+        :param refresh: Refresh the item metadata after the request.
+
         :returns: A Request if debug else a Response.
 
         Usage::
@@ -810,7 +824,12 @@ class Item(BaseItem):
         secret_key = secret_key or self.session.secret_key
         debug = bool(debug)
         headers = headers or {}
+        expect = expect or {}
         request_kwargs = request_kwargs or {}
+        if timeout:
+            request_kwargs["timeout"] = float(timeout)  # type: ignore
+        else:
+            request_kwargs["timeout"] = 60  # type: ignore
 
         _headers = self.session.headers.copy()
         _headers.update(headers)
@@ -830,6 +849,7 @@ class Item(BaseItem):
             access_key=access_key,
             secret_key=secret_key,
             append=append,
+            expect=expect,
             append_list=append_list,
             insert=insert)
         # Must use Session.prepare_request to make sure session settings
@@ -839,7 +859,8 @@ class Item(BaseItem):
             return prepared_request
         resp = self.session.send(prepared_request, **request_kwargs)
         # Re-initialize the Item object with the updated metadata.
-        self.refresh()
+        if refresh:
+            self.refresh()
         return resp
 
     # TODO: `list` parameter name shadows the Python builtin
@@ -876,7 +897,8 @@ class Item(BaseItem):
                     retries_sleep: int | None = None,
                     debug: bool = False,
                     validate_identifier: bool = False,
-                    request_kwargs: MutableMapping | None = None) -> Request | Response:
+                    request_kwargs: MutableMapping | None = None,
+                    set_scanner: bool = True) -> Request | Response:
         """Upload a single file to an item. The item will be created
         if it does not exist.
 
@@ -1039,7 +1061,8 @@ class Item(BaseItem):
                                 file_metadata=file_metadata,
                                 access_key=access_key,
                                 secret_key=secret_key,
-                                queue_derive=queue_derive)
+                                queue_derive=queue_derive,
+                                set_scanner=set_scanner)
             return request
 
         if debug:
@@ -1099,9 +1122,9 @@ class Item(BaseItem):
                 return response
             except HTTPError as exc:
                 try:
-                    msg = get_s3_xml_text(exc.response.content)
+                    msg = get_s3_xml_text(exc.response.content)  # type: ignore
                 except ExpatError:  # probably HTTP 500 error and response is invalid XML
-                    msg = ('IA S3 returned invalid XML '
+                    msg = ('IA S3 returned invalid XML '  # type: ignore
                            f'(HTTP status code {exc.response.status_code}). '
                            'This is a server side error which is either temporary, '
                            'or requires the intervention of IA admins.')
@@ -1129,7 +1152,8 @@ class Item(BaseItem):
                retries_sleep: int | None = None,
                debug: bool = False,
                validate_identifier: bool = False,
-               request_kwargs: dict | None = None) -> list[Request | Response]:
+               request_kwargs: dict | None = None,
+               set_scanner: bool = True) -> list[Request | Response]:
         r"""Upload files to an item. The item will be created if it
         does not exist.
 
@@ -1186,11 +1210,13 @@ class Item(BaseItem):
 
         responses = []
         file_index = 0
-        if queue_derive and total_files is None:
-            if checksum:
-                total_files = recursive_file_count(files, item=self, checksum=True)
-            else:
-                total_files = recursive_file_count(files, item=self, checksum=False)
+        headers = headers or {}
+        if (queue_derive or not headers.get('x-archive-size-hint')) and total_files == 0:
+            total_files, total_size = recursive_file_count_and_size(files,
+                                                                    item=self,
+                                                                    checksum=checksum)
+            if not headers.get('x-archive-size-hint'):
+                headers['x-archive-size-hint'] = str(total_size)
         file_metadata = None
         for f in files:
             if isinstance(f, dict):
@@ -1235,7 +1261,8 @@ class Item(BaseItem):
                                             retries_sleep=retries_sleep,
                                             debug=debug,
                                             validate_identifier=validate_identifier,
-                                            request_kwargs=request_kwargs)
+                                            request_kwargs=request_kwargs,
+                                            set_scanner=set_scanner)
                     responses.append(resp)
             else:
                 file_index += 1
@@ -1269,7 +1296,8 @@ class Item(BaseItem):
                                         retries_sleep=retries_sleep,
                                         debug=debug,
                                         validate_identifier=validate_identifier,
-                                        request_kwargs=request_kwargs)
+                                        request_kwargs=request_kwargs,
+                                        set_scanner=set_scanner)
                 responses.append(resp)
         return responses
 
